ExecuteRange = LibStub("AceAddon-3.0"):NewAddon("ExecuteRange", "AceConsole-3.0", "AceEvent-3.0" );

--GLOBALS and CONSTANTS--
local _G = _G --Global enviroment table provided by WoW Api
local Buttons = {};
local ValidSpellsTable = {["DISPATCH_ID"]=111240,
						  ["SHADOWBURN_ID"]=17877,
						  ["SHADOW_WORD_DEATH_ID"]=32379,
						  ["SHADOW_WORD_DEATH_GLYPHED_ID"]=129176,
						  ["TOUCH_OF_DEATH_ID"]=115080,
						  ["SOUL_REAPER_FROST_ID"]=130735,
						  ["SOUL_REAPER_UNHOLY_ID"]=130736,
						  ["SOUL_REAPER_BLOOD_ID"]=114866,
						  ["EXECUTE_ID"]=5308};
local DISPATCH_EXECUTE_RANGE = 35;
local SHADOWBURN_EXECUTE_RANGE = 20;
local SHADOW_WORD_DEATH_EXECUTE_RANGE = 20;
local EXECUTE_RANGE = 20;
local SOUL_REAPER_EXECUTE_RANGE = 35;
local PlayerClass = "";

local defaults = {
  profile = {
    showSpellAlert = true,
	enabled = true
  }
}
  
-- Called when the addon is loaded
function ExecuteRange:OnInitialize()

	-- Init DB
	self.db = LibStub("AceDB-3.0"):New("ExecuteRangeDB",defaults)

	local localizedClass, englishClass = UnitClass("player");
	PlayerClass = englishClass;
	if PlayerClass ~= "ROGUE" and PlayerClass ~="WARLOCK" and PlayerClass ~= "PRIEST" and PlayerClass ~= "MONK" and PlayerClass ~="DEATHKNIGHT" and PlayerClass ~="WARRIOR" then 
		self:Print("Your class is not Supported !! Addon will be disabled");
		ExecuteRange:Disable();
		return;
	end

	-- Registers the slash commands handler
	self:RegisterChatCommand("exrange", "SlashCommandHandler", true);

	if self.db.profile.enabled == false then
		ExecuteRange:Disable();
		return;
	end
end

-- Called when the addon is enabled
function ExecuteRange:OnEnable()

	-- Collects all the buttons of the UI in the global array Buttons
    Buttons = ExecuteRange:GetAllButtons()

	--Registers the UNIT_HEALTH event that fires when a unit's health changes
	self:RegisterEvent("UNIT_HEALTH");

	--Registers the PLAYER_TARGET_CHANGED event. This event is fired whenever the player's target is changed, including when the target is lost. 
	self:RegisterEvent("PLAYER_TARGET_CHANGED");

	--Registers the UNIT_AURA event. This event is fired when a buff, debuff, status, or item bonus was gained by or faded from an entity (player, pet, NPC, or mob.) 
	--Used to monitor monk's Touch of Death
	if PlayerClass == "MONK" then
		self:RegisterEvent("UNIT_AURA");
	end

	self:Print("To see a list of commands type /exrange help");
end

-- Called when the addon is disabled
function ExecuteRange:OnDisable()
    
end

--UNIT_HEALTH event handler
--@param eventName Placeholder parameter by Ace. The name of the event
--@param arg1 the UnitID that triggered the event. e.x. "target","focus"
function ExecuteRange:UNIT_HEALTH(eventName, arg1)
	if arg1 == "target" then
		ExecuteRange:ShowOrHideFlasher();
	end
end

--PLAYER_TARGET_CHANGED event handler
--@param eventName Placeholder parameter by Ace. The name of the event
--@param arg1 The way the target was changed. e.x. Escape, Left click
function ExecuteRange:PLAYER_TARGET_CHANGED(eventName, arg1)
	ExecuteRange:ShowOrHideFlasher();
end

--UNIT_AURA event handler
--@param eventName Placeholder parameter by Ace. The name of the event
--@param arg1 the UnitID that triggered the event. e.x. "target","focus"
function ExecuteRange:UNIT_AURA(eventName, arg1)
	if arg1 == "player" then
		ExecuteRange:ShowOrHideFlasher();
	end
end

--Main function. Shows or hides the glow/spell alert accordingly
function ExecuteRange:ShowOrHideFlasher()
	
	local health = UnitHealth("target");
	local maxHealth = UnitHealthMax("target"); 
	local percentage = (health/maxHealth) * 100;
	local executeRange = ExecuteRange:GetExecuteRange();
		
	if percentage <= executeRange and not UnitIsDead("target") and UnitCanAttack("player","target") then 
		-- Show Glow
		local foundButtons = ExecuteRange:GetValidButtons(Buttons);

		for id,button in pairs(foundButtons) do
			ActionButton_ShowOverlayGlow(button);
		end

		--Show Spell Alert
		if self.db.profile.showSpellAlert and table.getn(foundButtons) > 0  then
			ExecuteRange:ShowSpellAlert();
		end

	else
		-- Hide Glow
		local foundButtons = ExecuteRange:GetValidButtons(Buttons);
		for id,button in pairs(foundButtons) do
			ActionButton_HideOverlayGlow(button);
		end

		--Hide Spell Alert
		if self.db.profile.showSpellAlert then
			ExecuteRange:HideSpellAlert();
		end 
	end
end

--Gets the health percentage that execute procs depending on logged in class
function ExecuteRange:GetExecuteRange()
	if PlayerClass == "ROGUE" then
		return DISPATCH_EXECUTE_RANGE;
	elseif PlayerClass == "WARLOCK" then
		return SHADOWBURN_EXECUTE_RANGE;
	elseif PlayerClass == "PRIEST" then
		return SHADOW_WORD_DEATH_EXECUTE_RANGE;
	elseif PlayerClass == "DEATHKNIGHT" then
		return SOUL_REAPER_EXECUTE_RANGE;
	elseif PlayerClass == "WARRIOR" then
		return EXECUTE_RANGE;
	elseif PlayerClass == "MONK" then  
	--Monk is a special case. The percentage is not fixed. If the buff "Death Note" is active you can cast Touch of Death.
		name = UnitBuff("player","Death Note"); 
		if name ~=nill then -- if name is nil then "Death Note" is not active
			return 100;		--A execute range of 100 will always be higher than the target's life percentage, so the effect will be activated
		else 
			return 0;		--A execute range of 0 will always be lower than the target's life percentage, so the effect will NOT be activated
		end
	end
end

--Shows a Spell Alert Overlay appropriate to the logged in class
function ExecuteRange:ShowSpellAlert()
	if PlayerClass == "ROGUE" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DEATH.BLP", "RIGHT", 1, 255, 255, 255, false, true);
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DEATH.BLP", "LEFT", 1, 255, 255, 255, false, false);
	elseif PlayerClass == "WARLOCK" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DOOM.BLP", "RIGHT", 1, 255, 255, 255, false, true);
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DOOM.BLP", "LEFT", 1, 255, 255, 255, false, false);
	elseif PlayerClass == "PRIEST" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\NECROPOLIS.BLP", "BOTTOM", 1, 255, 255, 255, true, true);
	elseif PlayerClass == "DEATHKNIGHT" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\NECROPOLIS.BLP", "BOTTOM", 1, 255, 255, 255, true, true);
	elseif PlayerClass == "MONK" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\DENOUNCE.BLP", "TOP", 2, 255, 255, 255, false, false);
	elseif PlayerClass == "WARRIOR" then
		local id, specName, description, icon, background,role=GetSpecializationInfo(GetSpecialization());
		if specName == "Protection" then --if protection spec'ed, show alert on bottom.
			SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ULTIMATUM.BLP", "BOTTOM", 1, 255, 255,255, true, false);
		else
			SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ULTIMATUM.BLP", "TOP", 1, 255, 255,255, false, false);
		end
	end
end 

function ExecuteRange:HideSpellAlert()
		SpellActivationOverlay_HideOverlays(SpellActivationOverlayFrame, "EXECUTE_RANGE_OVERLAY");
end 

--Finds the buttons that have valid abilities e.x. "Dispatch", "Shadowburn" etc
--@param buttons The array of buttons to look into
function ExecuteRange:GetValidButtons(buttons)
	
	local foundButtons = {};
	for name,button in pairs(buttons) do
		local LAB = LibStub("LibActionButton-1.0", true);
		local type,spellID,subType; 
		if(LAB) then --bartender
			spellID = button:GetSpellId();
		else		--blizzard
			type,spellID,subType  = GetActionInfo(button.action);
		end
		if ExecuteRange:TableContainsItem(ValidSpellsTable,spellID) then
			table.insert(foundButtons,button);
		end
	end
	return foundButtons;
end

--Checks if an item is contained in a table
--@param tbl The table to check
--@param item The item to check if it exists in the table
function ExecuteRange:TableContainsItem(tbl, item)
    for key, value in pairs(tbl) do
        if value == item then return true end
    end
    return false
end

--Gets all the buttons of the UI. Currently Bartender and Blizzard UI are supported
function ExecuteRange:GetAllButtons()
	
	local LAB = LibStub("LibActionButton-1.0", true);
	local buttons ={};
	if(LAB) then
		self:Print("Barteder Detected");
		for button in pairs(LAB:GetAllButtons()) do
			buttons[button:GetName()] = button;
		end
	else
		self:Print("BlizzardUI Detected");
		ExecuteRange:GetBlizzardButtons("ActionButton",buttons);
		ExecuteRange:GetBlizzardButtons("BonusActionButton",buttons);
		ExecuteRange:GetBlizzardButtons("MultiBarRightButton",buttons);
		ExecuteRange:GetBlizzardButtons("MultiBarLeftButton",buttons);
		ExecuteRange:GetBlizzardButtons("MultiBarBottomRightButton",buttons);
		ExecuteRange:GetBlizzardButtons("MultiBarBottomLeftButton",buttons);
	end
	return buttons;
end

-- Collects Blizzard's UI buttons and adds them to the given buttons array.
--@param barName The name of the blizzard bar to collect buttons from. e.x. "ActionButton", "MultiBarRightButton"
--@param buttonsArray The array to insert into the buttons found.
--@param count The number of buttons of this type. Default 12.
function ExecuteRange:GetBlizzardButtons(buttonType, buttonsArray, count)
	for i=1,(count or 12) do
		if _G[buttonType .. i] ~= nil then
			buttonsArray[buttonType .. i] = _G[buttonType .. i];
		end
	end
end

-- Slash commands handler. 
--@param msg Contains the parameters passed after /exrange
--@param editbox Contains the chat window that was used to enter the command
function ExecuteRange:SlashCommandHandler(msg, editbox)
	if msg == nil or msg == "" or msg == "help" then 
		self:Print("/exrange help  -- This info");		
		self:Print("/exrange status  -- Current status of the addon");	
		self:Print("/exrange sa on  -- Turns spell alerts visual on");
		self:Print("/exrange sa off  -- Turns spell alerts visual off");
		self:Print("/exrange enable -- Enables the addon");		
		self:Print("/exrange disable -- Disables the addon");
	elseif msg == "sa on" then
		self.db.profile.showSpellAlert = true;
		self:Print("Spell alerts: on");
	elseif msg == "sa off" then
		self.db.profile.showSpellAlert = false;
		self:Print("Spell alerts: off");
	elseif msg == "enable" then
		self.db.profile.enabled = true;
		ExecuteRange:Enable();
		self:Print("Enabled");
	elseif msg == "disable" then
		self.db.profile.enabled = false;
		ExecuteRange:Disable();
		self:Print("Disabled");
	elseif msg == "status" then
		if self.db.profile.enabled then 
			self:Print("Enabled");
		else
			self:Print("Disabled");
		end
		if self.db.profile.showSpellAlert then
			self:Print("Spell alerts: on");
		else
			self:Print("Spell alerts: off");
		end
	else
		self:Print("Unknown command. To see a list of commands type /exrange help");		
	end	
end